precision mediump float;

uniform sampler2D u_Texture;
uniform sampler2D countryTexture;
varying vec2 v_TexCoord;
uniform float iTime;

#define mod3_ vec3(.1031, .22369, .13787)
#define PI 3.1415926359

vec3 hash3_3(vec3 p3) {
	p3 = fract(p3 * mod3_);
    p3 += dot(p3, p3.yxz + 120.0);
    // random3 has range 0 to 1
    vec3 random3 = fract(vec3((p3.x + p3.y) * p3.z, (p3.x+p3.z) * p3.y, (p3.y+p3.z) * p3.x));
    return normalize(-1. + 2. * random3);
}

float perlin_noise3(vec3 p) {
    vec3 pi = floor(p);
    vec3 pf = p - pi;

    // 5th order interpolant from Improved Perlin Noise
    vec3 pf3 = pf * pf * pf;
    vec3 pf4 = pf3 * pf;
    vec3 pf5 = pf4 * pf;
    vec3 w = 6. * pf5 - 15. * pf4 + 10. * pf3;

    return mix(
    	mix(
            mix(
                dot(pf - vec3(0, 0, 0), hash3_3(pi + vec3(0, 0, 0))),
                dot(pf - vec3(1, 0, 0), hash3_3(pi + vec3(1, 0, 0))),
                w.x),
            mix(
                dot(pf - vec3(0, 0, 1), hash3_3(pi + vec3(0, 0, 1))),
                dot(pf - vec3(1, 0, 1), hash3_3(pi + vec3(1, 0, 1))),
                w.x),
    	w.z),
        mix(
            mix(
                dot(pf - vec3(0, 1, 0), hash3_3(pi + vec3(0, 1, 0))),
                dot(pf - vec3(1, 1, 0), hash3_3(pi + vec3(1, 1, 0))),
                w.x),
            mix(
                dot(pf - vec3(0, 1, 1), hash3_3(pi + vec3(0, 1, 1))),
                dot(pf - vec3(1, 1, 1), hash3_3(pi + vec3(1, 1, 1))),
                w.x),
     	w.z),
	w.y);
}

vec2 pos3t2(vec3 pos){
	float r = length(pos);
	float Y = acos(pos.y / r) / PI;
	float X = atan(-pos.z/r, pos.x/r) / PI / 2.0;

	return vec2(X,Y);
}

vec3 pos2t3(vec2 pos){
	float X = sin(pos.y*PI) * cos(pos.x* PI*2.0);
	float Y = cos(pos.y*PI);
	float Z = -sin(pos.y*PI) * sin(pos.x* PI*2.0);
	return vec3(X,Y,Z);
}

vec2 Spiral(vec2 uv)
{
	float reps = 2.0;
	vec2 uv2 = fract(uv*reps);
	vec2 center = floor(fract(uv*reps)) + 0.5;
	vec2 delta = uv2 - center;
	float dist = length(delta);
	float angle = atan(delta.y, delta.x);
	float nudge = dist * 4.0;
	vec2 offset = vec2(delta.y, -delta.x);
	float blend = max(abs(delta.x), abs(delta.y))* 2.0;
	blend = clamp((0.5 - dist) * 2.0, 0.0, 1.0);
	blend = pow(blend, 1.5);
	offset *= clamp(blend, 0.0, 1.0);
	return uv + offset*vec2(1.0,1.0)*1.1+ vec2(iTime*-0.03, 0.0);
}

float cloud(vec3 p){
    vec3 t = p;
    p.x=t.x * cos(iTime*0.01) - t.z*sin(iTime*0.01);
    p.z=t.x * sin(iTime*0.01) + t.z* cos(iTime*0.01);
    vec2 uv = pos3t2(p);

    vec3 cp = pos2t3(Spiral(uv* 2.0) + Spiral(uv* 3.0));

    float c =  perlin_noise3(cp*vec3(8.0,16.0,8.0));
    c += perlin_noise3(cp*vec3(4.0,8.0,4.0))*2.0;
    c += perlin_noise3(cp*vec3(1.0,2.0,1.0))*3.0;
    c += perlin_noise3(cp*vec3(32.0,64.0,32.0));
    c -= perlin_noise3(p*1.5);
    c -= perlin_noise3(p*5.0);

    float s = abs((abs(p.y) - 0.5)*2.0);
	c += s;

    return max(c ,0.0);
}

void main()
{
    gl_FragColor = texture2D(u_Texture, v_TexCoord);

    float dis = 0.0;
    vec3 n;
    vec3 f;
    vec3 earthPos = vec3(v_TexCoord.x, v_TexCoord.y, 0.4);

    // "Water" color
    if (gl_FragColor.r >= 0.14 && gl_FragColor.r <= 0.15 &&
        gl_FragColor.g >= 0.14 && gl_FragColor.g <= 0.15 &&
        gl_FragColor.b >= 0.14 && gl_FragColor.b <= 0.15 &&
        gl_FragColor.a == 1.0) {

        gl_FragColor = vec4(0.04, 0.04, 0.04, 1.0);
    }
    // All the countries fill color
    else if (gl_FragColor.r >= 0.23 && gl_FragColor.r <= 0.24 &&
        gl_FragColor.g >= 0.23 && gl_FragColor.g <= 0.24 &&
        gl_FragColor.b >= 0.23 && gl_FragColor.b <= 0.24 &&
        gl_FragColor.a == 1.0) {

        gl_FragColor = vec4(0.16, 0.16, 0.17, 1.0);
    }
    // All the countries border color
    else if (gl_FragColor.r > 0.5 &&
        gl_FragColor.g > 0.5 &&
        gl_FragColor.b > 0.5 &&
        gl_FragColor.a == 1.0) {

        gl_FragColor = vec4(0.4, 0.4, 0.4, 1.0);
    }
    // Selected country fill color
    else if (gl_FragColor.rgb == vec3(0.0)) {
        gl_FragColor = vec4(0.13, 0.13, 0.13, 1.0);
    }

    // Country border
    vec4 fragCountryColor = texture2D(countryTexture, v_TexCoord);
    if (fragCountryColor.r >= 0.9 &&
        fragCountryColor.g >= 0.9 &&
        fragCountryColor.b >= 0.9) {

        fragCountryColor.rgb *= vec3(0.5, 0.5, 0.5);
        gl_FragColor = mix(gl_FragColor, fragCountryColor, 1.0);
    }
}