precision mediump float;

uniform sampler2D u_Texture;
varying vec2 v_TexCoord;
uniform float iTime;
uniform vec2 iResolution;
uniform vec4 iRect1;

float rand(vec2 n) {
    return fract(sin(dot(n, vec2(12.9898,12.1414))) * 83758.5453);
}

float noise(vec2 n) {
    const vec2 d = vec2(0.0, 1.0);
    vec2 b = floor(n);
    vec2 f = smoothstep(vec2(0.0), vec2(1.0), fract(n));
    return mix(mix(rand(b), rand(b + d.yx), f.x), mix(rand(b + d.xy), rand(b + d.yy), f.x), f.y);
}

vec3 ramp(float t) {
	return t <= .5 ? vec3( 1. - t * 1.4, .2, 1.05 ) / t : vec3( .3 * (1. - t) * 2., .2, 1.05 ) / t;
}
vec2 polarMap(vec2 uv, float shift, float inner) {

    uv = vec2(0.5) - uv;


    float px = 1.0 - fract(atan(uv.y, uv.x) / 6.28 + 0.25) + shift;
    float py = (sqrt(uv.x * uv.x + uv.y * uv.y) * (1.0 + inner * 2.0) - inner) * 2.0;

    return vec2(px, py);
}
float fire(vec2 n) {
    return noise(n) + noise(n * 2.1) * .6 + noise(n * 5.4) * .42;
}

float shade(vec2 uv, float t) {
    uv.x += uv.y < .5 ? 23.0 + t * .035 : -11.0 + t * .03;
    uv.y = abs(uv.y - .5);
    uv.x *= 35.0;

    float q = fire(uv - t * .013) / 2.0;
    vec2 r = vec2(fire(uv + q / 2.0 + t - uv.x - uv.y), fire(uv + q - t));

    return pow((r.y + r.y) * max(.0, uv.y) + .1, 4.0);
}

vec3 color(float grad) {

    float m2 = 0.15;
    grad =sqrt( grad);
    vec3 color = vec3(1.0 / (pow(vec3(0.5, 0.0, .1) + 2.61, vec3(2.0))));
    vec3 color2 = color;
    color = ramp(grad);
    color /= (m2 + max(vec3(0), color));

    return color;
}

float variation(vec2 v1, vec2 v2, float strength, float speed) {
	return sin(
        dot(normalize(v1), normalize(v2)) * strength + iTime * speed
    ) / 100.0;
}

void main()
{
    gl_FragColor = texture2D(u_Texture, v_TexCoord);

    // All the countries border - glowing effect
    if (gl_FragColor.r >= 0.2 &&
        gl_FragColor.g >= 0.2 &&
        gl_FragColor.b >= 0.2 &&
        gl_FragColor.a >= 0.2) {

        float evalR = (9.0 * iTime);
        float evalG = 1.0;
        float evalB = 1.0;
        float eval = (2.0 * iTime);

        float radius = 0.7555;
        vec2 center = vec2(0.5);

        vec2 diff = center - (v_TexCoord);
        float len = length(diff);
        float width = 0.09;

        len += variation(diff, vec2(0.0, 1.0), 50.0, 50.0);
        len -= variation(diff, vec2(1.0, 0.0), 50.0, 50.0);

        float r = 1.6;
        float omega = 2.0;

        vec3 col = vec3(r * (cos(omega * iTime) + 2.) / 2. - len, 0., 0.);
        gl_FragColor *= vec4(col, 1.0);

        float circle = smoothstep(radius-width, radius, len) - smoothstep(radius, radius+width, len);
        float m1 = 3.6;

        float t = iTime;
        vec2 uv = v_TexCoord / 10.0;
        float ff = 1.0 - uv.y;
        uv.x -= (20.0 / 10.0 - 1.0) / 2.0;
        vec2 uv2 = uv;
        uv2.y = 1.0 - uv2.y;
        uv = polarMap(uv, 1.3, m1);
        uv2 = polarMap(uv2, 1.9, m1);

        vec3 c1 = color(shade(uv, t)) * ff;
        vec3 c2 = color(shade(uv2, t)) * (1.0 - ff);

        gl_FragColor += vec4(ramp(iTime * 10.0), 1.0);

    }
    // Selected country fill
    else if (gl_FragColor.r == 0.0 &&
        gl_FragColor.g == 0.0 &&
        gl_FragColor.b == 0.0) {

        float eval = iTime * 0.1;
        float c = min(eval - (0.5 * floor(eval/0.5)), 0.3);
        c = max(c, 0.2);
        gl_FragColor.rgb = vec3(c);
    }
}