precision mediump float;

varying vec2 v_TexCoord;
uniform float iTime;
uniform vec2 iResolution;
uniform float isLandscape;

float sdParabola( in vec2 pos, in float wi, in float he )
{
    pos.x = abs(pos.x);

    float ik = wi*wi/he;
    float p = ik*(he-pos.y-0.5*ik)/3.0;
    float q = pos.x*ik*ik*0.25;
    float h = q*q - p*p*p;

    float x;
    if( h>0.0 ) // 1 root
    {
        float r = sqrt(h);
        x = pow(q+r,1.0/3.0) - pow(abs(q-r),1.0/3.0)*sign(r-q);
    }
    else        // 3 roots
    {
        float r = sqrt(p);
        x = 2.0*r*cos(acos(q/(p*r))/3.0);
    }

    x = min(x,wi);

    return length(pos-vec2(x,he-x*x/ik)) *
           sign(ik*(pos.y-he)+pos.x*pos.x);
}

vec2 rotate(vec2 v, float a) {
	float s = sin(radians(a));
	float c = cos(radians(a));
	mat2 m = mat2(c, -s, s, c);
	return m * v;
}

void main() {
    vec2 p = (2.0 * v_TexCoord ) - iResolution / iResolution;

    if (isLandscape == 1.0) {
        p = rotate(p, -90.0);
        p.x -= 1.02;
        p.y += 1.12;
    }

    // position
    float px = (isLandscape == 0.0) ? 0.0 : -1.023; // x position
    float py = (isLandscape == 0.0) ? -1.023 : 0.0; // y position
    float pw = (isLandscape == 0.0) ? 1.1 : 1.3;    // width

    // sdf
    float d = sdParabola( p-vec2(px,py), pw, 0.35 );
    float d2 = sdParabola( p-vec2(px,py-0.07), pw + 0.1, 0.44 );

    float ra = 0.01;
    d =  abs(d) - ra;

    // colorize
    vec4 col = (d>0.0) ? vec4(0.0) : vec4( 1.0);
    col = mix( col, vec4( 1.0), 1.0-smoothstep(0.0,0.01,abs(d)) );

    vec4 col2 =  vec4( 1.0);
    col2 = mix( col2, vec4( 0.0), smoothstep(0.0,0.01,abs(d2) - 0.01) );

    col2 -= col;

    float backd = sdParabola( p-vec2(px, py - 0.23), pw + 0.28, 0.56 );
    // colorize
    vec4 col3 = (backd>0.0) ? vec4(0.0705, 0.0705, 0.0705, 1.0) : vec4( 0.0);
    gl_FragColor = col3;

    if (col2.a > 0.0) {
        vec2 scrPt = (2.0 * iResolution.x * v_TexCoord - iResolution.xy) / iResolution.y * 10.0; // get NDC

        // Create rotation matrix based on time
        float theta = -iTime * 1.3 * 3.14159;
        float cosTheta, sinTheta;
        cosTheta = cos(theta);
        sinTheta = sin(theta);
        mat2 rot = mat2(cosTheta, -sinTheta, sinTheta, cosTheta);

        vec2 scrPtRot = rot * scrPt;

        // Parameterize radar shape
        float angle = 1.0 - (atan(scrPtRot.y, scrPtRot.x) / 18.2831 + 0.2);
        float falloff = length(scrPtRot);

        float angleFalloff = pow(angle, 25.0); // the exponent determines the radial width

        col2 *= vec4(0.45,0.8,0.25, 1.0);
        col2 *= vec4(vec3(angleFalloff * falloff) * vec3(0.55, 0.96, 0.3), 0);
        gl_FragColor += col2;
    }
}